using System;
using System.Runtime.Remoting;

namespace CallBackDAO
{
    /// <summary>
    /// This is the actual Notification object itself which contains the event handler
    /// </summary>
    public class Notification : MarshalByRefObject
	{
        public event EventHandler Notified;

		public  Notification()
		{
		}

        public virtual void OnNotified(EventArgs e)
        {
            if (Notified != null)
            {
                Notified(this,e);
            }
        }

        public void Update()
        {
            OnNotified(EventArgs.Empty);
        }
	}

    /// <summary>
    /// This is a factory that allows creation of a singleton using .NET Remoting
    /// </summary>
    public class NotificationFactory : MarshalByRefObject
    {
        private static Notification instance;

        public static Notification Instance
        {
            get
            {
                if (instance == null)
                {
                    if (instance == null)
                    {
                        ObjectHandle objectHandle = Activator.CreateInstance("CallBackDAO", "CallBackDAO.Notification");
                        instance = (Notification)objectHandle.Unwrap();
                    }
                }
                return instance;
            }
        }
    }

    /// <summary>
    /// This is the object that is exposed to the Java Client
    /// </summary>
    public class NotificationRemoting : MarshalByRefObject
    {
        public NotificationRemoting()
        {}

        public void Update()
        {
            Notification n = NotificationFactory.Instance;
            n.Update();
        }
    }
}
