using System;
using System.Collections;
using System.Data.SqlClient;
using System.Runtime.Remoting;

namespace CallBackDAO
{
    public delegate void UpdateEventHandler(object sender, EventArgs e);

    public class StockRecordDAO
    {
        private SqlConnection _conn = null;
        private String CONNECTION_STRING = "Data Source=localhost; User ID=testuser; Password=StrongPassword; Initial Catalog=SharedDB";
        public event UpdateEventHandler Updated;
        private Notification n = NotificationFactory.Instance;

        public StockRecordDAO()
        {
            // Configure the remoting listener
            RemotingConfiguration.Configure(@"remoting.config");

            // Configure the event handler for any updates from the Java client
            n.Notified += new EventHandler(IncomingNotification);

            // open the conection to the db - throw exception if can't open
            try
            {
                _conn = new SqlConnection(CONNECTION_STRING);
                _conn.Open();
            }
            catch (SqlException sqle)
            {
                throw new StockRecordDAOException(sqle);
            }
        }

        public StockRecord GetStockRecord(long ID)
        {
            // run the query
            SqlCommand cmd = new SqlCommand("SELECT * FROM ORDERTBL WHERE ID = "+ID,_conn);
            SqlDataReader reader = cmd.ExecuteReader();

            reader.Read();
            if (!reader.HasRows) throw new StockRecordDAOException("Stock Record not found");
            
            StockRecord recordToReturn = nextRowToRecord(reader);            
            reader.Close();
            return recordToReturn;
        }

        public ArrayList GetStockRecords(long accountID)
        {
            return SelectRecords("SELECT * FROM ORDERTBL WHERE ACCOUNT_ID = "+accountID);
        }

        public ArrayList GetStockRecords(String type, long accountID)
        {
            return SelectRecords("SELECT * FROM ORDERTBL WHERE TYPE = '"+type+"' AND ACCOUNT_ID = "+accountID);
        }

        public ArrayList GetStockRecords(String type, String ticker)
        {
            return SelectRecords("SELECT * FROM ORDERTBL WHERE TYPE = '"+type+"' AND TICKER = '"+ticker+"'");
        }

        public ArrayList GetStockRecords(long accountID, String ticker)
        {
            return SelectRecords("SELECT * FROM ORDERTBL WHERE ACCOUNT_ID = "+accountID+" AND TICKER = '"+ticker+"'");
        }
        
        public ArrayList GetStockRecords(String type, long accountID, String ticker)
        {
            return SelectRecords("SELECT * FROM ORDERTBL WHERE TYPE='"+type+"' AND ACCOUNT_ID = "+accountID+" AND TICKER = '"+ticker+"'");
        }

        public ArrayList GetAllStockRecords()
        {
            return SelectRecords("SELECT * FROM ORDERTBL");
        }

        private ArrayList SelectRecords(String selectString)
        {   
            SqlCommand cmd = new SqlCommand(selectString,_conn);
            SqlDataReader reader = cmd.ExecuteReader();

            if (!reader.HasRows) throw new StockRecordDAOException("Stock Record not found");

            ArrayList returnedRecords = new ArrayList();
            while (reader.Read())
            {
                returnedRecords.Add(nextRowToRecord(reader));
            }

            reader.Close();

            return returnedRecords;
        }

        public void AddStockRecord(StockRecord record)
        {
            SqlCommand cmd = new SqlCommand("INSERT ORDERTBL (TYPE,ACCOUNT_ID,TICKER,QTY,PRICE) VALUES ('"+record.Type+"',"+record.AccountID+",'"+record.Ticker+"',"+record.Qty+","+record.Price+")",_conn);
            int recordsAffected = cmd.ExecuteNonQuery();

            if (recordsAffected != 1) throw new StockRecordDAOException("Add record may have failed.");
        }

        private StockRecord nextRowToRecord(SqlDataReader reader)
        {
            return new StockRecord(reader.GetInt64(0),reader.GetString(1),reader.GetInt64(2),reader.GetString(3),reader.GetInt32(4),reader.GetSqlMoney(5).ToDouble());
        }

        public virtual void OnUpdated(EventArgs e)
        {
            if (Updated != null)
            {
                Updated(this,e);
            }
        }

        public void IncomingNotification(object o, EventArgs e)
        {
            // Trigger the DAO Event Handler that will be consumed by the business logic
            OnUpdated(EventArgs.Empty);
        }
	}


    public class StockRecordDAOException : Exception
    {
        public StockRecordDAOException() : base()
        {
        }

        public StockRecordDAOException(Exception e) : base(e.ToString())
        { 
        }

        public StockRecordDAOException(String message) : base(message)
        {
        }
    }
}
